VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "IteratorContent"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' _C_ Content structure
Private Enum ContentStruct
  [_First] = 1
  
  S_C_TASK_TYPE = 1
  S_C_STATUS = 2
  S_C_CONTENT_NAME = 3
  S_C_UPDATE_STATUS = 4
  S_C_CHANGE_SCORE = 5
  S_C_BIBLIO_NAME = 6
  S_C_DEFINITION = 7
  S_C_IS_IMMUTABLE = 8
  S_C_OBJECT_TYPE = 9
  S_C_MARKERS = 10
  S_C_TAGS = 11
  S_C_AUTHOR = 12
  S_C_SUPERVISOR = 13
  S_C_EXECUTOR = 14
  S_C_EDITOR = 15
  S_C_RESPONSIBLE = 16
  S_C_DEPARTMENT = 17
  S_C_TARGET_DATE = 18
  S_C_SOURCE = 19
  S_C_ELECTRON_BRE = 20
  S_C_MAIN_PAGE = 21
  S_C_IS_GENERAL = 22
  S_C_ACTUALIZE_PERIOD = 23
  S_C_AGE_RESTRICTION = 24
  S_C_PRIORITY = 25
  S_C_ARTICLE_TYPE = 26
  S_C_DATE_EXCHANGE = 27
  S_C_DATE_EES1 = 28
  S_C_DATE_EX_TOOLS = 29
  S_C_DATE_EES2 = 30
  S_C_EXPERT = 31
  S_C_CONTRACT = 32
  S_C_COMMENT = 33
  S_C_TASK_ID = 34
  S_C_CONTENT_NAME_DB = 35
  S_C_TASK_NAME = 36
  S_C_PARENT_ID = 37
  
  [_Last] = 37
End Enum

Public row_ As Long

Private data_ As Excel.Worksheet

Public Sub Init(target As Excel.Worksheet, Optional tRow& = DATA_FIRST_ROW)
  Set data_ = target
  row_ = tRow
End Sub

Public Function Increment(Optional inc& = 1)
  If row_ + inc > 0 Then _
    row_ = row_ + inc
End Function

Public Function GoFirst()
  row_ = DATA_FIRST_ROW
End Function

Public Function GoLast()
  row_ = GetLastRow
End Function

Public Function GoEmpty()
  Call GoLast
  Call Increment
End Function

Public Function IsDone() As Boolean
  IsDone = row_ > GetLastRow
End Function

Public Function IsCanceled() As Boolean
  IsCanceled = Status = STATUS_CANCELED
End Function

Public Function HasContent() As Boolean
  Dim sContent$: sContent = ContentNameDB
  HasContent = sContent <> NO_CONTENT_PLACEHOLDER And sContent <> NO_MEDIA_PLACEHOLDER
End Function

Public Function RemoveRow()
  Call data_.Rows(row_).Delete
End Function

Public Function FindTaskID(sID$) As Boolean
  If sID = "" Then _
    Exit Function
  Dim iFound As Excel.Range:
  Set iFound = data_.Columns(S_C_TASK_ID).Find(sID, LookAt:=xlWhole)
  FindTaskID = Not iFound Is Nothing
  If FindTaskID Then _
    row_ = iFound.Row
End Function

Public Function FindContentName(sName$, sType$) As Boolean
  If sName = "" Then _
    Exit Function
  Dim iFound As Excel.Range:
  Set iFound = data_.Columns(S_C_CONTENT_NAME).Find(sName, LookAt:=xlWhole)
  If iFound Is Nothing Then _
    Exit Function
  If data_.Cells(iFound.Row, S_C_TASK_TYPE) <> sType Then _
    Exit Function
  row_ = iFound.Row
  FindContentName = True
End Function

Public Function SyncCSVTasks(iInput As IteratorCSVTasks)
  TaskType = iInput.TaskType
  Status = iInput.Status
  ContentName = iInput.ContentName
  ContentNameDB = iInput.ContentNameDB
  Supervisor = iInput.Supervisor
  Executor = iInput.Executor
  TargetDate = iInput.TargetDate
  TaskID = iInput.TaskID
  TaskName = iInput.TaskName
  ParentID = iInput.ParentID
End Function

Public Function SyncCSVContent(iInput As IteratorCSVContent)
  Call ValidateValue(S_C_BIBLIO_NAME, iInput.BiblioName)
  Call ValidateValue(S_C_CHANGE_SCORE, iInput.ChangeScore)
  Call ValidateValue(S_C_DEPARTMENT, iInput.Department)
  Call ValidateValue(S_C_RESPONSIBLE, iInput.Responsible)
  Call ValidateValue(S_C_EDITOR, iInput.Editor)
  Call ValidateValue(S_C_DEFINITION, iInput.Definition)
  Call ValidateValue(S_C_OBJECT_TYPE, iInput.ObjectType)
  Call ValidateListValue(S_C_MARKERS, iInput.Markers)
  Call ValidateListValue(S_C_TAGS, iInput.Tags)
  Call ValidateValue(S_C_SOURCE, iInput.Source)
  Call ValidateValue(S_C_ELECTRON_BRE, iInput.ElectronBre)
  Call ValidateValue(S_C_MAIN_PAGE, iInput.MainPage)
  Call ValidateValue(S_C_IS_GENERAL, iInput.IsGeneral)
  Call ValidateValue(S_C_ACTUALIZE_PERIOD, iInput.ActualizePeriod)
  Call ValidateValue(S_C_AGE_RESTRICTION, iInput.AgeRestriction)
  Call ValidateValue(S_C_AUTHOR, iInput.Author)
  If UpdateStatus = T_UPD_ONCE Then _
    UpdateStatus = T_UPD_UNDEF
  Call RecalculateStatus
End Function

Public Function SyncContent(iInput As IteratorContent)
  TaskType = iInput.TaskType
  Status = iInput.Status
  ContentName = iInput.ContentName
  Supervisor = iInput.Supervisor
  Executor = iInput.Executor
  TargetDate = iInput.TargetDate
  
  IsImmutable = iInput.IsImmutable
  IsMain = iInput.IsMain
  IsGeneral = iInput.IsGeneral
  IsBRE = iInput.IsBRE
  
  UpdateStatus = iInput.UpdateStatus
  If iInput.BiblioName <> "" Then BiblioName = iInput.BiblioName
  If iInput.ChangeScore <> "" Then ChangeScore = iInput.ChangeScore
  If iInput.Definition <> "" Then Definition = iInput.Definition
  If iInput.ObjectType <> "" Then ObjectType = iInput.ObjectType
  If iInput.Markers <> "" Then Markers = iInput.Markers
  If iInput.Author <> "" Then Author = iInput.Author
  If iInput.Tags <> "" Then Tags = iInput.Tags
  If iInput.Editor <> "" Then Editor = iInput.Editor
  If iInput.Responsible <> "" Then Responsible = iInput.Responsible
  If iInput.Department <> "" Then Department = iInput.Department
  If iInput.Source <> "" Then Source = iInput.Source
  If iInput.ActualizePeriod <> "" Then ActualizePeriod = iInput.ActualizePeriod
  If iInput.AgeRestriction <> "" Then AgeRestriction = iInput.AgeRestriction
  If iInput.Priority <> "" Then Priority = iInput.Priority
  If iInput.ArticleType <> "" Then ArticleType = iInput.ArticleType
  If iInput.Expert <> "" Then Expert = iInput.Expert
  If iInput.Contract <> "" Then Contract = iInput.Contract
  If iInput.ContentNameDB <> "" Then ContentNameDB = iInput.ContentNameDB
  If iInput.TaskID <> "" Then TaskID = iInput.TaskID
  If iInput.TaskName <> "" Then TaskName = iInput.TaskName
  If iInput.ParentID <> "" Then ParentID = iInput.ParentID
  
  Call RecalculateStatus
End Function

Public Function HasBlanks() As Boolean
  HasBlanks = True
  
  If BiblioName = "" Then _
    Exit Function
  If Definition = "" Then _
    Exit Function
  If Markers = "" Then _
    Exit Function
  If ObjectType = "" Then _
    Exit Function
  If Supervisor = "" Then _
    Exit Function
  If Editor = "" Then _
    Exit Function
    
  HasBlanks = False
End Function

Public Function IsIgnored() As Boolean
  Dim oldStatus As TUpdateStatus: oldStatus = UpdateStatus
  IsIgnored = oldStatus = T_UPD_COMPLETE Or oldStatus = T_UPD_IGNORE
End Function

Public Function NeedsUpdate() As Boolean
  Dim oldStatus As TUpdateStatus: oldStatus = UpdateStatus
  NeedsUpdate = oldStatus = T_UPD_AUTO Or oldStatus = T_UPD_ONCE Or oldStatus = T_UPD_ALWAYS
End Function

Public Function RecalculateStatus() As TUpdateStatus
  Dim oldStatus As TUpdateStatus: oldStatus = UpdateStatus
  RecalculateStatus = oldStatus
  If oldStatus = T_UPD_ONCE Or oldStatus = T_UPD_ALWAYS Or oldStatus = T_UPD_IGNORE Then _
    Exit Function
  If TaskID = "" Then
    UpdateStatus = T_UPD_UNDEF
  ElseIf ContentNameDB = " " Or Status = "" Then
    UpdateStatus = T_UPD_UNDEF
  ElseIf HasBlanks Then
    UpdateStatus = T_UPD_AUTO
  Else
    UpdateStatus = T_UPD_COMPLETE
  End If
  RecalculateStatus = UpdateStatus
End Function

' ======== Property Get =========
Public Property Get TaskType() As String
  TaskType = data_.Cells(row_, S_C_TASK_TYPE)
End Property

Public Property Get Status() As String
  Status = data_.Cells(row_, S_C_STATUS)
End Property

Public Property Get ContentName() As String
  ContentName = data_.Cells(row_, S_C_CONTENT_NAME)
End Property

Public Property Get UpdateStatus() As TUpdateStatus
  UpdateStatus = UpdateStatusFromText(data_.Cells(row_, S_C_UPDATE_STATUS))
End Property

Public Property Get ChangeScore() As String
  ChangeScore = data_.Cells(row_, S_C_CHANGE_SCORE)
End Property

Public Property Get BiblioName() As String
  BiblioName = data_.Cells(row_, S_C_BIBLIO_NAME)
End Property

Public Property Get Definition() As String
  Definition = data_.Cells(row_, S_C_DEFINITION)
End Property

Public Property Get IsImmutable() As Boolean
  IsImmutable = data_.Cells(row_, S_C_IS_IMMUTABLE) = BOOL_TEXT_YES
End Property

Public Property Get ObjectType() As String
  ObjectType = data_.Cells(row_, S_C_OBJECT_TYPE)
End Property

Public Property Get Markers() As String
  Markers = data_.Cells(row_, S_C_MARKERS)
End Property

Public Property Get Tags() As String
  Tags = data_.Cells(row_, S_C_TAGS)
End Property

Public Property Get Author() As String
  Author = data_.Cells(row_, S_C_AUTHOR)
End Property

Public Property Get Supervisor() As String
  Supervisor = data_.Cells(row_, S_C_SUPERVISOR)
End Property

Public Property Get Executor() As String
  Executor = data_.Cells(row_, S_C_EXECUTOR)
End Property

Public Property Get Editor() As String
  Editor = data_.Cells(row_, S_C_EDITOR)
End Property

Public Property Get Responsible() As String
  Responsible = data_.Cells(row_, S_C_RESPONSIBLE)
End Property

Public Property Get Department() As String
  Department = data_.Cells(row_, S_C_DEPARTMENT)
End Property

Public Property Get TargetDate() As String
  TargetDate = data_.Cells(row_, S_C_TARGET_DATE)
End Property

Public Property Get Source() As String
  Source = data_.Cells(row_, S_C_SOURCE)
End Property

Public Property Get IsBRE() As Boolean
  IsBRE = data_.Cells(row_, S_C_ELECTRON_BRE) = BOOL_TEXT_YES
End Property

Public Property Get IsMain() As Boolean
  IsMain = data_.Cells(row_, S_C_MAIN_PAGE) = BOOL_TEXT_YES
End Property

Public Property Get IsGeneral() As Boolean
  IsGeneral = data_.Cells(row_, S_C_IS_GENERAL) = BOOL_TEXT_YES
End Property

Public Property Get ActualizePeriod() As String
  ActualizePeriod = data_.Cells(row_, S_C_ACTUALIZE_PERIOD)
End Property

Public Property Get AgeRestriction() As String
  AgeRestriction = data_.Cells(row_, S_C_AGE_RESTRICTION)
End Property

Public Property Get Priority() As String
  Priority = data_.Cells(row_, S_C_PRIORITY)
End Property

Public Property Get ArticleType() As String
  ArticleType = data_.Cells(row_, S_C_ARTICLE_TYPE)
End Property

Public Property Get DateExchange() As String
  DateExchange = data_.Cells(row_, S_C_DATE_EXCHANGE)
End Property

Public Property Get DateEES1() As String
  DateEES1 = data_.Cells(row_, S_C_DATE_EES1)
End Property

Public Property Get DateTools() As String
  DateTools = data_.Cells(row_, S_C_DATE_EX_TOOLS)
End Property

Public Property Get DateEES2() As String
  DateEES2 = data_.Cells(row_, S_C_DATE_EES2)
End Property

Public Property Get Expert() As String
  Expert = data_.Cells(row_, S_C_EXPERT)
End Property

Public Property Get Contract() As String
  Contract = data_.Cells(row_, S_C_CONTRACT)
End Property

Public Property Get Comment() As String
  Comment = data_.Cells(row_, S_C_COMMENT)
End Property

Public Property Get TaskID() As String
  TaskID = data_.Cells(row_, S_C_TASK_ID)
End Property

Public Property Get ContentNameDB() As String
  ContentNameDB = data_.Cells(row_, S_C_CONTENT_NAME_DB)
End Property

Public Property Get TaskName() As String
  TaskName = data_.Cells(row_, S_C_TASK_NAME)
End Property

Public Property Get ParentID() As String
  ParentID = data_.Cells(row_, S_C_PARENT_ID)
End Property

' ==== Property Let ====
Public Property Let TaskID(newVal$)
  Dim oldVal$: oldVal = data_.Cells(row_, S_C_TASK_ID)
  data_.Cells(row_, S_C_TASK_ID) = newVal
  If newVal <> "" And (oldVal <> newVal Or data_.Cells(row_, S_C_TASK_TYPE).Hyperlinks.Count = 0) Then
    Call XLUpdateHyperlink(data_.Cells(row_, S_C_TASK_TYPE), URL_PREFIX_TASK & newVal)
    If HasContent Then
      Call XLUpdateHyperlink(data_.Cells(row_, S_C_CONTENT_NAME), URL_PREFIX_CONTENT & newVal)
    Else
      Call data_.Cells(row_, S_C_CONTENT_NAME).Hyperlinks.Delete
    End If
  End If
End Property

Public Property Let TaskType(newVal$)
  data_.Cells(row_, S_C_TASK_TYPE) = newVal
End Property

Public Property Let Status(newVal$)
  data_.Cells(row_, S_C_STATUS) = newVal
End Property

Public Property Let ContentName(newVal$)
  data_.Cells(row_, S_C_CONTENT_NAME) = newVal
End Property

Public Property Let UpdateStatus(newVal As TUpdateStatus)
  data_.Cells(row_, S_C_UPDATE_STATUS) = UpdateStatusToText(newVal)
End Property

Public Property Let ChangeScore(newVal$)
  data_.Cells(row_, S_C_CHANGE_SCORE) = newVal
End Property

Public Property Let BiblioName(newVal$)
  data_.Cells(row_, S_C_BIBLIO_NAME) = newVal
End Property

Public Property Let Definition(newVal$)
  data_.Cells(row_, S_C_DEFINITION) = newVal
End Property

Public Property Let IsImmutable(newVal As Boolean)
  data_.Cells(row_, S_C_IS_IMMUTABLE) = IIf(newVal, BOOL_TEXT_YES, BOOL_TEXT_NO)
End Property

Public Property Let ObjectType(newVal$)
  data_.Cells(row_, S_C_OBJECT_TYPE) = newVal
End Property

Public Property Let Markers(newVal$)
  data_.Cells(row_, S_C_MARKERS) = newVal
End Property

Public Property Let Tags(newVal$)
  data_.Cells(row_, S_C_TAGS) = newVal
End Property

Public Property Let Author(newVal$)
  data_.Cells(row_, S_C_AUTHOR) = newVal
End Property

Public Property Let Supervisor(newVal$)
  data_.Cells(row_, S_C_SUPERVISOR) = newVal
End Property

Public Property Let Executor(newVal$)
  data_.Cells(row_, S_C_EXECUTOR) = newVal
End Property

Public Property Let Editor(newVal$)
  data_.Cells(row_, S_C_EDITOR) = newVal
End Property

Public Property Let Responsible(newVal$)
  data_.Cells(row_, S_C_RESPONSIBLE) = newVal
End Property

Public Property Let Department(newVal$)
  data_.Cells(row_, S_C_DEPARTMENT) = newVal
End Property

Public Property Let TargetDate(newVal$)
  data_.Cells(row_, S_C_TARGET_DATE) = newVal
End Property

Public Property Let Source(newVal$)
  data_.Cells(row_, S_C_SOURCE) = newVal
End Property

Public Property Let IsBRE(newVal As Boolean)
  data_.Cells(row_, S_C_ELECTRON_BRE) = IIf(newVal, BOOL_TEXT_YES, BOOL_TEXT_NO)
End Property

Public Property Let IsMain(newVal As Boolean)
  data_.Cells(row_, S_C_MAIN_PAGE) = IIf(newVal, BOOL_TEXT_YES, BOOL_TEXT_NO)
End Property

Public Property Let IsGeneral(newVal As Boolean)
  data_.Cells(row_, S_C_IS_GENERAL) = IIf(newVal, BOOL_TEXT_YES, BOOL_TEXT_NO)
End Property

Public Property Let ActualizePeriod(newVal$)
  data_.Cells(row_, S_C_ACTUALIZE_PERIOD) = newVal
End Property

Public Property Let AgeRestriction(newVal$)
  data_.Cells(row_, S_C_AGE_RESTRICTION) = newVal
End Property

Public Property Let Priority(newVal$)
  data_.Cells(row_, S_C_PRIORITY) = newVal
End Property

Public Property Let ArticleType(newVal$)
  data_.Cells(row_, S_C_ARTICLE_TYPE) = newVal
End Property

Public Property Let DateExchange(newVal$)
  data_.Cells(row_, S_C_DATE_EXCHANGE) = newVal
End Property

Public Property Let DateEES1(newVal$)
  data_.Cells(row_, S_C_DATE_EES1) = newVal
End Property

Public Property Let DateTools(newVal$)
  data_.Cells(row_, S_C_DATE_EX_TOOLS) = newVal
End Property

Public Property Let DateEES2(newVal$)
  data_.Cells(row_, S_C_DATE_EES2) = newVal
End Property

Public Property Let Expert(newVal$)
  data_.Cells(row_, S_C_EXPERT) = newVal
End Property

Public Property Let Contract(newVal$)
  data_.Cells(row_, S_C_CONTRACT) = newVal
End Property

Public Property Let ContentNameDB(newVal$)
  data_.Cells(row_, S_C_CONTENT_NAME_DB) = newVal
End Property

Public Property Let TaskName(newVal$)
  data_.Cells(row_, S_C_TASK_NAME) = newVal
  If data_.Cells(row_, S_C_IS_IMMUTABLE) = "" Then
     If newVal Like " *" Then
      data_.Cells(row_, S_C_IS_IMMUTABLE) = BOOL_TEXT_YES
    Else
      data_.Cells(row_, S_C_IS_IMMUTABLE) = BOOL_TEXT_NO
    End If
  End If
End Property

Public Property Let ParentID(newVal$)
  data_.Cells(row_, S_C_PARENT_ID) = newVal
End Property

' =======
Private Function GetLastRow() As Long
  GetLastRow = data_.Cells(data_.Rows.Count, 1).End(xlUp).Row
End Function

Private Function ColorCell(nColumn&, nColor&)
  data_.Cells(row_, nColumn).Interior.Color = nColor
End Function

Private Function ValidateValue(nColumn&, portalValue$)
  Dim sValue$: sValue = data_.Cells(row_, nColumn)
  If sValue = portalValue Then _
    Exit Function
  
  If sValue = "" Then
    data_.Cells(row_, nColumn) = portalValue
    Exit Function
  End If
  
  If portalValue = "" Then
    Call ColorCell(nColumn, RGB(142, 169, 219))
  Else
    Call ColorCell(nColumn, RGB(255, 151, 151))
  End If
End Function

Private Function ValidateListValue(nColumn&, portalValue$)
  Dim sValue$: sValue = data_.Cells(row_, nColumn)
  If CheckListsEqual(sValue, portalValue) Then _
    Exit Function
  
  If sValue = "" Then
    data_.Cells(row_, nColumn) = portalValue
    Exit Function
  End If
  
  If portalValue = "" Then
    Call ColorCell(nColumn, RGB(142, 169, 219))
  Else
    Call ColorCell(nColumn, RGB(255, 151, 151))
  End If
End Function
