Attribute VB_Name = "MainImpl"
Option Private Module
Option Explicit

Public Function AccessContent() As DB_Content
  Static s_Content As DB_Content
  
  If s_Content Is Nothing Then
    Set s_Content = New DB_Content
    Call s_Content.Init(ThisWorkbook.Worksheets(SHEET_CONTENT), ThisWorkbook.Worksheets(SHEET_ATTRIBUTES))
  End If
  
  Set AccessContent = s_Content
End Function

Public Function AccessWorkers() As DB_Workers
  Static s_Workerks As DB_Workers
  
  If s_Workerks Is Nothing Then
    Set s_Workerks = New DB_Workers
    Call s_Workerks.Init(ThisWorkbook.Worksheets(SHEET_OPTIONS).ListObjects(TABLE_WORKERS))
  End If
  
  Set AccessWorkers = s_Workerks
End Function

Public Function AccessConfig() As InfoConfig
  Set AccessConfig = New InfoConfig
  Call AccessConfig.Init(ThisWorkbook.Worksheets(SHEET_CONFIG), AccessContent)
End Function

Public Function ClearData()
  Call XLShowAllData(ThisWorkbook.Sheets(SHEET_CONTENT))
  Call ThisWorkbook.Sheets(SHEET_CONTENT).UsedRange.Offset(1, 0).Rows.EntireRow.Delete
  
  Call XLShowAllData(ThisWorkbook.Sheets(SHEET_ATTRIBUTES))
  Call ThisWorkbook.Sheets(SHEET_ATTRIBUTES).UsedRange.Offset(1, 0).Rows.EntireRow.Delete
End Function

Public Function ImportTasksFromCSV(iInput As IteratorCSVTasks, iOutput As DB_Content)
  Call CSE_ProgressBar.Init("   CSV", maxVal:=iInput.CountRows)
  Call CSE_ProgressBar.ShowModeless
  
  Call iOutput.EnsureDataVisible
  Call iOutput.ImportCSVTasks(iInput)
  
  Call Unload(CSE_ProgressBar)
End Function

Public Function ImportContentFromCSV(iInput As IteratorCSVContent, iOutput As DB_Content)
  Call CSE_ProgressBar.Init("   CSV", maxVal:=iInput.CountRows)
  Call CSE_ProgressBar.ShowModeless
  
  Call iOutput.EnsureDataVisible
  Call iOutput.ImportCSVContent(iInput)
  
  Call Unload(CSE_ProgressBar)
End Function

Public Function ImportDataFromDB(iInput As DB_Content, iOutput As DB_Content)
  Call CSE_ProgressBar.Init(" ", maxVal:=iInput.Count)
  Call CSE_ProgressBar.ShowModeless
  
  Call iInput.EnsureDataVisible
  Call iOutput.EnsureDataVisible
  Call iOutput.ImportDB(iInput)
  
  Call Unload(CSE_ProgressBar)
End Function

Public Function PortalUpdate(iConfig As InfoConfig) As Boolean
  PortalUpdate = False
  Dim sExec$: sExec = EXPORTER_EXECUTABLE & " " & EXPORTER_CONFIG_FILE
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(ThisWorkbook.Path & "\" & EXPORTER_EXECUTABLE) Then
    Call UserInteraction.ShowMessage(EM_MISSING_EXEC, EXPORTER_EXECUTABLE)
    Exit Function
  End If
  
  Dim iShell As New WshShell
  iShell.CurrentDirectory = ThisWorkbook.Path
  On Error GoTo REPORT_EXEC
  If iShell.Run(sExec, WaitOnReturn:=True) <> 0 Then
REPORT_EXEC:
    Call UserInteraction.ShowMessage(EM_CANNOT_EXEC, sExec)
    Exit Function
  End If
  On Error GoTo 0
  
  Dim sOutput1$: sOutput1 = iConfig.OutputFileTasks
  If Not fso.FileExists(sOutput1) Then
    Call UserInteraction.ShowMessage(EM_MISSING_FILE, sExec)
    Exit Function
  End If
  
  Dim sOutput2$: sOutput2 = iConfig.OutputFileContent
  If iConfig.ScanContent And Not fso.FileExists(sOutput2) Then
    Call UserInteraction.ShowMessage(EM_MISSING_FILE, sExec)
    Exit Function
  End If
  PortalUpdate = True
End Function

Public Function CheckListsEqual(list1$, list2$) As Boolean
  CheckListsEqual = False
  Dim items1 As Variant: items1 = VBA.Split(list1, ";")
  Dim items2 As Variant: items2 = VBA.Split(list2, ";")
  If UBound(items1) <> UBound(items2) Then _
    Exit Function
    
  Dim it1 As Variant
  Dim it2 As Variant
  Dim flagExists As Boolean
  For Each it1 In items1
    flagExists = False
    
    For Each it2 In items2
      If it1 = it2 Then
        flagExists = True
        Exit For
      End If
    Next it2
    
    If Not flagExists Then _
      Exit Function
  Next it1
  
  For Each it2 In items2
    flagExists = False
    
    For Each it1 In items1
      If it1 = it2 Then
        flagExists = True
        Exit For
      End If
    Next it1
    
    If Not flagExists Then _
      Exit Function
  Next it2

  CheckListsEqual = True
End Function
